"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = loader;

var _fs = _interopRequireDefault(require("fs"));

var _path = _interopRequireDefault(require("path"));

var _loaderUtils = require("loader-utils");

var _jsYaml = _interopRequireDefault(require("js-yaml"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function getLocaleFiles(folder, filename) {
  const regExp = /([^\\]+)\.([^\.]+)\.(\w+)$/;

  try {
    let files = _fs.default.readdirSync(folder);

    let result = [];
    files.forEach(file => {
      let filepath = _path.default.join(folder, file);

      let stat = _fs.default.statSync(filepath);

      if (stat && !stat.isDirectory() && regExp.test(file)) {
        let [name, c, locale, ext] = regExp.exec(file);
        result.push({
          name,
          locale,
          ext
        });
      }
    });
    return result;
  } catch (err) {
    return null;
  }

  return null;
}

function readAndConvert(path, json = false) {
  const content = _fs.default.readFileSync(path, 'utf8');

  const data = _jsYaml.default.safeLoad(content);

  return json ? data : JSON.stringify(data, undefined, '\t');
}

function stringify(o) {
  return JSON.stringify(o).replace(/\u2028/g, '\\u2028').replace(/\u2029/g, '\\u2029').replace(/\\/g, '\\\\');
}

function generateCode(params) {
  const files = getLocaleFiles(params.resourcePath, params.name);
  let code = '',
      value = null;

  if (files) {
    let list = files.filter(({
      locale
    }) => params.locales.includes(locale)).map(({
      locale,
      name
    }) => ({
      [locale]: readAndConvert(_path.default.resolve(params.resourcePath, name), true)
    }));
    value = stringify(Object.assign({}, ...list));
  }

  code = `function (Component) {
  Component.options.__i18n = Component.options.__i18n || [];
  ${!!value} && Component.options.__i18n.push('${value.replace(/\u0027/g, '\\u0027')}');
}\n`;
  return code;
}

function loader(content, sourceMap) {
  const options = (0, _loaderUtils.getOptions)(this) || {};

  const {
    name
  } = _path.default.parse(this.resourcePath);

  const resourcePath = _path.default.resolve(_path.default.dirname(this.resourcePath), options.localesDir || "");

  try {
    this.cacheable && this.cacheable();
    const params = Object.assign({}, options, {
      resourcePath,
      name
    });
    this.callback(null, `module.exports = ${generateCode(params)}`, sourceMap);
  } catch (err) {
    this.emitError(err.message);
    this.callback(err);
  }
}